<?php

namespace Drupal\api_tokens\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\api_tokens\ApiTokenManager;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Controller routines for API tokens routes.
 */
class ApiTokenController extends ControllerBase {

  /**
   * The API token manager.
   *
   * @var \Drupal\api_tokens\ApiTokenManager
   */
  protected $apiTokenManager;

  /**
   * Constructs a new ApiTokenController.
   *
   * @param \Drupal\api_tokens\ApiTokenManager $api_token_manager
   *   The API token manager.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   */
  public function __construct(ApiTokenManager $api_token_manager, ModuleHandlerInterface $module_handler) {
    $this->apiTokenManager = $api_token_manager;
    $this->moduleHandler = $module_handler;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('plugin.manager.api_token'),
      $container->get('module_handler')
    );
  }

  /**
   * Displays the API tokens overview page.
   *
   * @return array
   *   A renderable array.
   */
  public function adminOverview() {
    $header = [
      $this->t('Token'),
      $this->t('Synopsis'),
      $this->t('Provider'),
    ];

    $rows = [];
    foreach ($this->apiTokenManager->getDefinitions() as $id => $definition) {
      $plugin = $this->apiTokenManager->createInstance($id, ['']);
      $provider = $plugin->getProvider();
      $rows[] = [
        'token' => [
          'data' => [
            '#type' => 'item',
            '#title' => $plugin->getLabel(),
            '#description' => $plugin->getDescription(),
            '#description_display' => 'after',
          ],
        ],
        'synopsis' => [
          'data' => $this->buildSynopsis($plugin),
        ],
        'provider' => [
          'data' => [
            '#type' => 'item',
            '#title' => $this->moduleHandler->getName($provider),
            '#description' => $this->t('Machine name: @provider', [
              '@provider' => $provider,
            ]),
            '#description_display' => 'after',
          ],
        ],
      ];
    }

    $build['table'] = array(
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#empty' => $this->t('There are no API tokens registered.'),
    );

    return $build;
  }

  /**
   * Renders the API token synopsis.
   *
   * @param \Drupal\api_tokens\Plugin\ApiTokenInterface $plugin
   *   The API token plugin instance.
   *
   * @return array
   *   A renderable array.
   */
  protected function buildSynopsis(\Drupal\api_tokens\Plugin\ApiTokenInterface $plugin) {
    $parts = [];
    foreach ($plugin->getBuildMethodReflector()->getParameters() as $param) {
      $name = $param->getName();
      if (!$param->isOptional()) {
        $name = "<strong>$name</strong>";
      }
      $parts[] = "<em>$name</em>";
    }
    $params = $parts ? '[' . implode(', ', $parts) . ']' : '';
    $build = ['#markup' => '<code>[api:' . $plugin->getId() . "$params/]</code>"];

    return $build;
  }

}
