<?php

namespace Drupal\api_tokens\Plugin;

use Drupal\Core\Plugin\PluginBase;
use Drupal\Core\Cache\RefinableCacheableDependencyTrait;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Component\Serialization\Json;

/**
 * Provides a base class for the API token plugins.
 *
 * @see \Drupal\api_tokens\Annotation\ApiToken
 * @see \Drupal\api_tokens\ApiTokenManager
 * @see \Drupal\api_tokens\Plugin\ApiTokenInterface
 * @see plugin_api
 */
abstract class ApiTokenBase extends PluginBase implements ApiTokenInterface {
  use RefinableCacheableDependencyTrait;

  /**
   * The API token string.
   *
   * @var string
   */
  protected $token;

  /**
   * The API token parameters.
   *
   * @var array
   */
  protected $params;

  /**
   * The API token parameters hash.
   *
   * @var string
   */
  protected $hash;

  /**
   * The API token render context.
   *
   * @var string[]
   */
  protected static $context = [];

  /**
   * Whether the recursive rendering occured.
   *
   * If in the process of API token rendering the same API token is met, we
   * could get an infinite rendering loop. When the recursive rendering occurs
   * the \Drupal\api_tokens\Plugin\ApiTokenInterface::getFallback() method
   * result will be used as the API token output.
   *
   * @var bool
   */
  protected $recursion;

  /**
   * The API token build method name.
   *
   * @var string
   */
  protected $buildMethod = 'build';

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $params = $configuration[0];
    $this->token = "[api:$plugin_id$params/]";
    $this->params = $params ? Json::decode($params) : [];
    $this->hash = $this->params ? hash('crc32b', serialize($this->params)) : '';
    $this->recursion = in_array("$this->pluginId:$this->hash", $this->getContext());
  }

  /**
   * {@inheritdoc}
   */
  public function getLabel() {
    return $this->pluginDefinition['title'];
  }

  /**
   * {@inheritdoc}
   */
  public function getDescription() {
    return $this->pluginDefinition['description'];
  }

  /**
   * {@inheritdoc}
   */
  public function getProvider() {
    return $this->pluginDefinition['provider'];
  }

  /**
   * {@inheritdoc}
   */
  public function getId() {
    return $this->pluginId;
  }

  /**
   * {@inheritdoc}
   */
  public function getToken() {
    return $this->token;
  }

  /**
   * {@inheritdoc}
   */
  public function getParameters() {
    return $this->params;
  }

  /**
   * {@inheritdoc}
   */
  public function getHash() {
    return $this->hash;
  }

  /**
   * {@inheritdoc}
   */
  public static function getContext() {
    return self::$context;
  }

  /**
   * {@inheritdoc}
   */
  public function getBuildMethodReflector() {
    $exists = method_exists($this, $this->buildMethod);
    return $exists ? new \ReflectionMethod($this, $this->buildMethod) : NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function validateToken() {
    $reflector = $this->getBuildMethodReflector();
    if (!$reflector) {
      $this->warn('ApiToken plugin "@label" has no "@method" method.', [
        '@label' => $this->getLabel(),
        '@method' => $this->buildMethod,
      ]);
      return FALSE;
    }

    if (!is_array($this->params)) {
      $this->warn('API token "@token" has invalid parameters format.', [
        '@token' => $this->token,
      ]);
      return FALSE;
    }

    if ($reflector->getNumberOfRequiredParameters() > count($this->params)) {
      $this->warn('API token "@token" has not enough parameters.', [
        '@token' => $this->token,
      ]);
      return FALSE;
    }

    $args = [];
    foreach ($reflector->getParameters() as $index => $param) {
      $use_default = $param->isOptional() && !isset($this->params[$index]);
      $args[$param->getName()] = $use_default ? $param->getDefaultValue() : $this->params[$index];
    }
    if (!$this->validate($this->params, $args)) {
      $this->warn('API token "@token" has invalid parameters.', [
        '@token' => $this->token,
      ]);
      return FALSE;
    }

    return TRUE;
  }

  /**
   * Validates the API token parameters.
   *
   * This validation must be context-independent. For example, if some parameter
   * is expected to be an entity ID, check only whether it is a valid entity ID,
   * but don't check the entity existence/access (these must be checked in the
   * build method).
   *
   * @param mixed[] $params
   *   An array of indexed API token parameters. For example, if we have the API
   *   token [api:example[123, ["param1", "param2"]]/], and plugin's build
   *   method argument definition is: ...($id, $options). The $params array will
   *   be:
   *   @code
   *   array(
   *     0 => 123,
   *     1 => array('param1', 'param2'),
   *   );
   *   @endcode
   *
   * @param array $args
   *   An array of named API token parameters. For example, if we have the API
   *   token [api:example[123, ["param1", "param2"]]/], and plugin's build
   *   method argument definition is: ...($id, $options). The $args array will
   *   be:
   *   @code
   *   array(
   *     'id' => 123,
   *     'options' => array('param1', 'param2'),
   *   );
   *   @endcode
   *
   * @return bool
   *   TRUE if the API token parameters were validated successfully, FALSE
   *   otherwise.
   *
   * @see \Drupal\api_tokens\Plugin\ApiTokenInterface::validateToken();
   */
  public function validate(array $params, array $args) {
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function process() {
    if (!$this->recursion) {
      $this->pushContext();
      $build = call_user_func_array(array($this, $this->buildMethod), $this->params);
      $this->alterBuild($build, clone $this);
      \Drupal::service('renderer')->renderPlain($build);
      $this->popContext();
      $this->addCacheTags(['api_token', "api_token:$this->pluginId"]);
      $this->addCacheableDependency(CacheableMetadata::createFromRenderArray($build));
      $build = [
        '#markup' => $build['#markup'],
        '#attached' => $build['#attached'],
      ];
    }
    else {
      $build = $this->getFallback();
      $this->warn('Recursion detected while rendering @token API token.', [
        '@token' => $this->token,
      ]);
    }

    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public static function lazyBuild($id, $params) {
    return \Drupal::service('plugin.manager.api_token')->createInstance($id, [$params])->process();
  }

  /**
   * Alters the API token build.
   *
   * @param array &$build
   *   The render array that is being created.
   * @param \Drupal\api_tokens\Plugin\ApiTokenInterface $plugin
   *   The API token plugin instance.
   */
  public function alterBuild(array &$build, ApiTokenInterface $plugin) {
    \Drupal::service('module_handler')->alter('api_token_build', $build, $plugin);
  }

  /**
   * {@inheritdoc}
   */
  public function getFallback() {
    return ['#markup' => (string) $this->fallback()];
  }

  /**
   * Returns a string to replace the API token with in cases of validation fail
   * of recursive rendering detection.
   *
   * @return string
   */
  public function fallback() {
    return '';
  }

  /**
   * Pushes the current API token to the context.
   */
  protected function pushContext() {
    self::$context[] = "$this->pluginId:$this->hash";
  }

  /**
   * Pops out the last item from the context.
   */
  protected function popContext() {
    array_pop(self::$context);
  }

  /**
   * Logs a warning message.
   *
   * @param string $text
   *   Message text.
   * @param array $args
   *   (optional) An associative array of replacements to make after
   *   translation.
   */
  protected function warn($text, $args = array()) {
    $message = $this->t($text, $args);
    \Drupal::logger('api_tokens')->warning($message);
    //trigger_error($message, E_USER_WARNING);
  }

  /**
   * {@inheritdoc}
   */
  public function _build() { }

}
