<?php

namespace Drupal\api_tokens\Plugin;

use Drupal\Component\Plugin\PluginInspectionInterface;
use Drupal\Core\Cache\RefinableCacheableDependencyInterface;

/**
 * Defines the interface for API token plugins.
 *
 * API tokens are discovered through annotations, which may contain the
 * following definition properties:
 * - id: (required) The API token ID.
 * - title: (required) The administrative label of the API token.
 * - description: The administrative description of the API token.
 *
 * @see \Drupal\api_tokens\Annotation\ApiToken
 * @see \Drupal\api_tokens\ApiTokenManager
 * @see \Drupal\api_tokens\Plugin\ApiTokenBase
 * @see plugin_api
 */
interface ApiTokenInterface extends PluginInspectionInterface, RefinableCacheableDependencyInterface {

  /**
   * Returns the administrative label of the API token.
   *
   * @return string
   */
  public function getLabel();

  /**
   * Returns the administrative description of the API token.
   *
   * @return string
   */
  public function getDescription();

  /**
   * Returns the name of the provider that owns this API token.
   *
   * @return string
   */
  public function getProvider();

  /**
   * Returns the API token ID.
   *
   * E.g. 'example' for [api:example[123, ["param1", "param2"]]/] token.
   *
   * @return string
   */
  public function getId();

  /**
   * Returns the API token string.
   *
   * E.g. '[api:example[123, ["param1", "param2"]]/]'.
   *
   * @return string
   */
  public function getToken();

  /**
   * Returns the API token parameters.
   *
   * Parameters parsed from the API token parameters string, e.g.:
   * @code
   * array(123, array('param1', 'param2'));
   * @endcode
   * for [api:example[123, ["param1", "param2"]]/] token.
   *
   * @return array
   */
  public function getParameters();

  /**
   * Returns the API token parameters hash.
   *
   * Used for render cache keys generation and context generation.
   *
   * @return string
   */
  public function getHash();

  /**
   * Returns the API token render context.
   *
   * If an API token (1) rendering was invoked from output generated by another
   * API token (2), render context for the token (1) will contain the token (2).
   * For example, the output of [api:token2/] contains an [api:token1/], so when
   * the "token1" will be rendered its context will be:
   * @code
   * array('[api:token2/]');
   * @endcode
   *
   * When called from hook_api_token_build_alter() context will contain the
   * token itself.
   *
   * @return string[]
   *   An array of API token strings.
   */
  public static function getContext();

  /**
   * Returns plugin's build method reflection object.
   *
   * @return \ReflectionMethod
   */
  public function getBuildMethodReflector();

  /**
   * Validates the API token.
   *
   * The API token is validated only when processed by the "API Tokens" filter,
   * after that the validation result is cached by the filter.
   * If the API token is rendered for the first time or its cache is
   * invalidated, the API token is built without this validation.
   * So this validation must be context-independent.
   *
   * Validation may include the following checks:
   * - the build method of the API token plugin exists;
   * - the parameters string of the API token is an empty string, or represents
   *   a valid JSON-encoded array;
   * - the API token has enough parameters for the build method;
   * - the API token parameters check. Fail validation, if specific parameter is
   *   invalid in any context. For example, if some parameter is expected to be
   *   an entity ID, check only whether it is a valid entity ID, but don't check
   *   the entity existence/access (these must be checked in the build method).
   *
   * @return bool
   *   TRUE if the API token was validated successfully, FALSE otherwise.
   */
  public function validateToken();

  /**
   * Processes the API token.
   *
   * Returns processed result of the plugin's build method.
   *
   * @return array
   *   A renderable array for the API token output.
   */
  public function process();

  /**
   * The #lazy_builder callback called from the "API Tokens" filter.
   *
   * Must instantiate the API token plugin and return the result of the
   * \Drupal\api_tokens\Plugin\ApiTokenInterface:process() method.
   *
   * @param $id
   *   The API token ID.
   * @param $params
   *   Parameters part of the API token string.
   *   JSON-encoded parameters array, e.g. '[123, ["param1", "param2"]]' for
   *   [api:example[123, ["param1", "param2"]]/] token.
   *
   * @return array
   *   A renderable array for the API token output.
   */
  public static function lazyBuild($id, $params);

  /**
   * Returns an output to replace the API token with in cases of validation fail
   * of recursive rendering detection.
   *
   * @return array
   *   A renderable array.
   */
  public function getFallback();

  /**
   * Returns the API token output.
   *
   * This is a dummy method. Each API token plugin must have its own build
   * method.
   *
   * Renderable array may contain cacheability metadata and attachments on any
   * level. Additional cacheability metadata can be added to the API token using
   * \Drupal\Core\Cache\RefinableCacheableDependencyInterface methods of the API
   * token plugin.
   *
   * @return array
   *   A renderable array.
   */
  public function _build(/*...*/);

}
