<?php

namespace Drupal\api_tokens\Plugin\Filter;

use Drupal\filter\Plugin\FilterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\filter\FilterProcessResult;

/**
 * Provides API tokens filter.
 *
 * @Filter(
 *   id = "api_tokens",
 *   title = @Translation("API Tokens"),
 *   description = @Translation("Render API tokens."),
 *   type = Drupal\filter\Plugin\FilterInterface::TYPE_MARKUP_LANGUAGE,
 *   settings = {
 *     "unregistered_tokens_behavior" = "cutout"
 *   }
 * )
 */
class ApiTokens extends FilterBase {

  /**
  * The API token pattern.
  */
  const PATTERN = '@\[api:([0-9a-z_-]+)(\[.*?\])?/]@';

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $form['unregistered_tokens_behavior'] = array(
      '#type' => 'radios',
      '#title' => $this->t('Unregistered API tokens process behavior'),
      '#default_value' => $this->settings['unregistered_tokens_behavior'],
      '#options' => [
        'cutout' => $this->t('Cut out'),
        'ignore' => $this->t('Ignore'),
      ],
    );

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function process($text, $langcode) {
    $result = new FilterProcessResult($text);
    if (preg_match_all(self::PATTERN, $text, $matches)) {
      list($tokens, $ids, $args) = $matches;
      $service = \Drupal::service('plugin.manager.api_token');
      $replacements = $tokens;
      foreach ($ids as $index => $id) {
        $definition = $service->getDefinition($id, FALSE);
        if ($definition) {
          $arg = $args[$index];
          $plugin = $service->createInstance($id, [$arg]);
          if ($plugin->validateToken()) {
            $plugin->process();
            $cache = [
              'contexts' => $plugin->getCacheContexts(),
              'tags' => $plugin->getCacheTags(),
              'max-age' => $plugin->getCacheMaxAge(),
            ];
            if (!$plugin->getContext()) {
              $cache['keys'] = ['api_token', $id, $plugin->getHash()];
            }
            $result->addAttachments([
              'placeholders' => [
                $tokens[$index] => [
                  '#lazy_builder' => [$definition['class'] . '::lazyBuild', [$id, $arg]],
                  '#cache' => $cache,
                ],
              ],
            ]);
          }
          else {
            $replacements[$index] = render($plugin->getFallback());
          }
        }
        elseif ('cutout' == $this->settings['unregistered_tokens_behavior']) {
          $replacements[$index] = '';
        }
      }
      $text = str_replace($tokens, $replacements, $text);
      $result->setProcessedText($text);
    }

    return $result;
  }

  /**
   * {@inheritdoc}
   */
  public function tips($long = FALSE) {
    return $this->t('Replace API tokens.');
  }

}
